#! /bin/env python3
# -*- coding: utf-8 -*-

'''
Python library to send lyrics from OpenLP to OBS-Studio.
'''

class DummyOutput:
	def send(self, d):
		print(d)

async def debug():
	inst = olp2obs.core.Olp2Obs('localhost')
	inst.flg_debug = 1
	await inst.loop()

def main():
	import sys
	import optparse
	parser = optparse.OptionParser()
	parser.add_option('--openlp', dest='openlp_uri', default='localhost', help='set host and port of OpenLP')
	parser.add_option('--obsws', dest='obs_host_port', default='localhost:4444', help='set host and port for obs-websocket')
	parser.add_option('-v', '--verbose', dest='verbose', action='count', default=0, help='verbose level')
	parser.add_option('--dummy-output', dest='dummy_output', action='store_true', help='just output to stdout (debug purpose)')
	parser.add_option('--ja-defaults', dest='ja_defaults', action='store_true', help='use Japanese default filter chain')
	parser.add_option('--pango', dest='pango', action='store_true', help='output pango XML')
	opts, args = parser.parse_args(sys.argv)

	import olp2obs.olp2txt
	import olp2obs.core
	import olp2obs.filters
	inst = olp2obs.core.Olp2Obs(opts.openlp_uri)
	inst.flg_debug = opts.verbose
	if opts.dummy_output:
		inst.obs = DummyOutput()
	if opts.ja_defaults:
		inst.filters = [(olp2obs.filters.filter_ja_pango, opts)]
	if opts.obs_host_port:
		from olp2obs.txt2obs import Text2Obs
		hp = opts.obs_host_port.split(':')
		inst.obs = Text2Obs(hp[0], int(hp[1]) if len(hp)>2 else 4444)
		inst.obs.sourceName = hp[-1]

	import asyncio
	loop = asyncio.get_event_loop()
	if opts.obs_host_port:
		inst.obs.loop = loop
	loop.run_until_complete(inst.loop())
