#! /bin/env python3
# -*- coding: utf-8 -*-

import asyncio
import websockets
import re
import olp2obs.olp2txt

class Olp2Obs:

	hide_at_exit = True

	def __init__(self, url_olp, host_obs=None, name_obs_src=None):
		self.olp = olp2obs.olp2txt.Olp2Text(url_olp)
		self.host_obs = host_obs
		self.name_obs_src = name_obs_src
		self.obs = None
		self.filters = [(olp2obs.olp2txt.filter_default, 'text'),]

	def run_filters(self, d):
		for func, opt in self.filters:
			d = func(d, opt)
			if not d:
				return None
		return d

	async def loop(self):
		d_prev = -1
		while True:
			d = None
			try:
				d = await self.olp.get_text()
				if d:
					d = self.run_filters(d)
			except KeyboardInterrupt:
				break
			except:
				if self.flg_debug:
					import traceback
					traceback.print_exc()
				d = None

			try:
				if d != d_prev:
					if self.flg_debug: print(d)
					await self.obs.send(d)
					d_prev = d
			except KeyboardInterrupt:
				break
			except:
				if self.flg_debug:
					import traceback
					traceback.print_exc()

			await asyncio.sleep(1)

		if self.hide_at_exit:
			try:
				await self.obs.send('')
			except:
				if self.flg_debug:
					import traceback
					traceback.print_exc()
		await self.olp.disconnect()
