#! /bin/env python3
# -*- coding: utf-8 -*-

import re

r_ascii = re.compile('[0-9a-zA-Z: ()]')
def _my_count(s):
	ret = 0
	for c in s:
		ret += 1 if r_ascii.match(c) else 2
	return ret

_r_nowrap = re.compile('[。、」]')
_r_alnum = re.compile('[A-Za-z0-9,.]')
_r_wrap = re.compile('[「]$')

def _split(data, n_th):
	data = re.sub("[ \t　]+", ' ', data)
	line = ''
	lines = []
	n_line = 0
	c_prev = ''
	for c in data:
		n = _my_count(c)
		if \
				n_line + n <= n_th or \
				_r_nowrap.match(c) or \
				(_r_alnum.match(c) and _r_alnum.match(c_prev)) :
			line += c
			n_line += n
		else:
			t = _r_wrap.search(line)
			if t:
				t = t.start()
				lines.append(line[0:t])
				line = line[t:] + c
				n_line = _my_count(c)
			else:
				lines.append(line)
				line = c
				n_line = n
		c_prev = c
	if len(line):
		lines.append(line)
	return lines

def _split_long_line(v, n):
	v1 = []
	for s in v:
		v2 = _split(s, n)
		for t in v2:
			v1.append(t)
	return v1

def textfilter_linebreak(s, n):
	v = s.split('\n')
	v = _split_long_line(v, n)
	i=0; s=''
	for s1 in v:
		i1 = _my_count(s1)
		if i + i1<n:
			s = s+' ' + s1 if len(s) else s1
			i += i1 + 1
		else:
			s += '\n' + s1 if len(s) else s1
			i = i1
	return s

_dakuon  = 'かがきぎくぐけげこご'
_dakuon += 'さざしじすずせぜそぞ'
_dakuon += 'ただちぢつづてでとど'
_dakuon += 'はばひびふぶへべほぼ'
_dakuon += 'うゔウヴ'
_dakuon += 'カガキギクグケゲコゴ'
_dakuon += 'サザシジスズセゼソゾ'
_dakuon += 'タダチヂツヅテデトド'
_dakuon += 'ハバヒビフブヘベホボ'
_handakuon = u'はぱひぴふぷへぺほぽ'
_handakuon += u'ハパヒピフプヘペホポ'
_x19 = ' *×[1-9]'

def textfilter_dakuon(s):
	import re
	s = re.sub('０', '0', s)
	s = re.sub('１', '1', s)
	s = re.sub('２', '2', s)
	s = re.sub('３', '3', s)
	s = re.sub('４', '4', s)
	s = re.sub('５', '5', s)
	s = re.sub('６', '6', s)
	s = re.sub('７', '7', s)
	s = re.sub('８', '8', s)
	s = re.sub('９', '9', s)
	s = re.sub(_x19, '', s)
	for i in range(0, len(_dakuon), 2):
		s = re.sub(_dakuon[i]+'\u3099', _dakuon[i+1], s)
	for i in range(0, len(_handakuon), 2):
		s = re.sub(_handakuon[i]+'\u309a', _handakuon[i+1], s)
	return s

def textfilter_zen2han(s):
	s = re.sub(r'！', '! ', s)
	s = re.sub(r"[　\t ]+", ' ', s)
	s = re.sub(r' $', '', s)
	s = re.sub(r'([0-9])~([0-9])', r'\1-\2', s)
	s = re.sub(r'[””]', '"', s)
	return s

def textfilter_empty_lines(s):
	s = re.sub('\n[ \n\t]*\n', '\n', s)
	s = re.sub('\n$', '', s)
	return s

def textfilter_remove_tags(s):
	tags = set()
	for t in s.split('<')[1:]:
		t = t.split('>')[0]
		tags.add('<%s>'%t)
	for t in tags:
		s = s.replace(t, '')
	return s

def htmlfilter_pango(s):
	import re
	s = re.sub('<(rp|rt)>[^<>]*</(rp|rt)>', '', s)
	s = re.sub('<br>', '\n', s)
	tags = set()
	for t in s.split('<')[1:]:
		t = t.split('>')[0]
		t = '<%s>'%t
		if t!='<sup>' and t!='</sup>':
			tags.add(t)
	for t in tags:
		s = s.replace(t, '')
	s = s.replace('&nbsp;', ' ')
	return s

def textfilter_lf_bibleja(s):
	return s.replace('／', '\n')

def filter_ja_pango(data, opts):
	is_lyrics = True if re.match(r'^[^0-9]+[1-9][0-9]*$', data['tag']) else False
	s = htmlfilter_pango(data['html'])
	s = textfilter_zen2han(s)
	if opts.pango:
		s = s.replace('\uff0d\uff0d', '<span letter_spacing="-10000">&#8212;&#8212;</span>')
	s = textfilter_dakuon(s)
	s = textfilter_lf_bibleja(s)
	s = textfilter_empty_lines(s)
	if is_lyrics:
		s = textfilter_linebreak(s, 50) # TODO: threshold of line-break
	if not opts.pango:
		s = textfilter_remove_tags(s)
		s = s.replace('&lt;', '<')
		s = s.replace('&gt;', '>')
		s = s.replace('&amp;', '&')
	return s
