#! /bin/env python3
# -*- coding: utf-8 -*-

import asyncio
import aiohttp
import aiohttp.client_exceptions
import websockets
import re
import http.client
import json


def is_blank(poll):
	is_blank = poll['results']['blank']
	is_theme = poll['results']['theme']
	is_display = poll['results']['display']
	if is_blank or is_theme or is_display:
		return True
	return False

def filter_slides_to_slide(poll, data):
	i_slide = poll['results']['slide']
	return data['results']['slides'][i_slide]

_re_is_movie_file = re.compile(r'.*\.(mp4|mov|mpeg|flv)$')

def filter_filename(data):
	if 'img' in data:
		return None
	h = data['html']
	t = data['text']
	if h==t and _re_is_movie_file.match(t):
		return None
	return data

def filter_default(data, opt='text'):
	if not data:
		return None
	return data[opt]

class Olp2Text:
	_default_olp_port = 4316
	def set_host(self, url_olp):
		m = re.match(r'^http://([^:/]*):([1-9][0-9]*)/$', url_olp)
		if m:
			self.host = m.group(1)
			self.port = m.group(2)
			return
		m = re.match(r'^http://([^:/]*)/$', url_olp)
		if m:
			self.host = m.group(1)
			self.port = self._default_olp_port
			return
		m = re.match(r'^([^:/]*):([1-9][0-9]*)$', url_olp)
		if m:
			self.host = m.group(1)
			self.port = m.group(2)
			return
		self.host = url_olp
		self.port = self._default_olp_port

	def __init__(self, url_olp):
		self.conn = None
		self.set_host(url_olp)
		self.poll_prev = None

	async def connect(self):
		if self.conn != None:
			raise ConnectionFailure('Server is already connected.')
		self.conn = aiohttp.TCPConnector()

	async def disconnect(self):
		await self.conn.close()

	async def olp_get(self, url):
		# print('opening URL %s'%url)
		if not self.conn:
			await self.connect()
		async with aiohttp.request('GET', url, connector=self.conn) as res:
			txt = await res.text()
			# print('retured text: "%s"'%txt)
			return txt

	async def olp_poll(self):
		url = 'http://%s:%d/api/poll'%(self.host, self.port)
		return await self.olp_get(url)

	async def olp_text(self):
		url = 'http://%s:%d/api/controller/live/text'%(self.host, self.port)
		return await self.olp_get(url)

	async def get_text(self):
		try:
			while True:
				res = await self.olp_poll()
				res = json.loads(res)
				if res != self.poll_prev:
					self.poll_prev = res
					break
				await asyncio.sleep(0.1)
			if is_blank(res):
				return None
			if not res:
				return None
			data = await self.olp_text()
			data = json.loads(data)
			return filter_filename(filter_slides_to_slide(res, data))
		except aiohttp.client_exceptions.ClientOSError:
			return None
